#!/usr/bin/env python
# encoding: utf-8
"""
Routines to do assorted difference and comparison calculations and statistics

Created by rayg Apr 2009.
Copyright (c) 2009 University of Wisconsin SSEC. All rights reserved.
"""

import os, sys, logging
from numpy import *

LOG = logging.getLogger(__name__)


def _missing(x,missing_value=None):
    if missing_value is not None:
        return isnan(x) | (x==missing_value)
    return isnan(x)


def statistics(a, b, epsilon=0., missing_value=None):
    """return dictionary of similarity statistics
    stats not including 'nan' in name exclude nans in either arrays
    """
    shape = a.shape
    aflat = a.flatten()
    bflat = b.flatten()
    n_a_missing = 0
    n_b_missing = 0
    if missing_value is not None:
        mvm = (missing_value==aflat)
        n_a_missing = mvm.sum()
        aflat[mvm] = nan
        mvm = (missing_value==bflat)
        n_b_missing = mvm.sum()
        bflat[mvm] = nan
    dflat = bflat-aflat
    del aflat
    del bflat
    n = len(dflat)
    dflat = array(dflat, float64)
    a_nans = isnan(a.flatten())
    b_nans = isnan(b.flatten())
    fin = isfinite(dflat) 
    
    dflat = dflat[fin]
    perc = dflat/a.flatten()[fin]*100.
    rms = sqrt(sum(dflat*dflat)/float(n))  # should n be len(dflat)?
    outside_epsilon = abs(dflat)>=epsilon
    n_o_e = outside_epsilon.sum()
    
    return { 'mean_percent_change': perc.mean(),
             'max_percent_change': abs(perc).max(),
             'mean_diff': dflat.mean(),
             'std_diff': std(dflat),
             'max_diff': abs(dflat).max(),
             'rms_diff': rms,
             'finite_count': len(dflat),
             'a_missing_count': n_a_missing,
             'b_missing_count': n_b_missing,
             'outside_epsilon_count': n_o_e,
             'outside_epsilon_fraction': n_o_e / float(n),
             'max_count': n,
             'a_nan_count': a_nans.sum(),
             'b_nan_count': b_nans.sum(),
             'a_and_b_nan_count': (a_nans & b_nans).sum(),
             'shape': shape
             }



if __name__=='__main__':
    import doctest
    doctest.testmod()
